from . import interactions
from .colors import Color

AppCommandInter = interactions.ApplicationCommandInteraction
"""Alias for ApplicationCommandInteraction"""
Colour = Color
"""Alias for Color class as per British english"""
